import { marked } from 'marked';
import type { Post } from '@blog/model/Post';
import type { PostMeta } from '@blog/model/PostMeta';
import * as yamlFront from 'yaml-front-matter';

export async function openPostMarkdownFile(filename: string): Promise<string> {
  const file = Bun.file(filename);
  return file.text();
}

type PostMetaWithRawContent = {
  meta: PostMeta,
  content: string
};

export function parsePostMetadata(post: string): PostMetaWithRawContent {
  const parsedData = yamlFront.loadFront(post);

  return {
    meta: {
      title: parsedData.title,
      description: parsedData.description,
      date: new Date(parsedData.date),
      draft: parsedData?.draft || false,
      tags: parsedData.tags
    },
    content: parsedData.__content
  };
}

export async function readPostMarkdown(filename: string): Promise<Post> {
  const contents = await openPostMarkdownFile(filename);
  const post = parsePostMetadata(contents);
  const parsedPost = await marked.parse(post.content);

  return {
    meta: post.meta,
    content: post.content,
    html: parsedPost
  };
}

